@@ -1,12 +1,13 @@
 #!/usr/bin/perl -w
 
-my $VERSION = '0.5.1';
+my $VERSION = '0.6.0';
 
 use strict;
 # use the ToolKit
 use Tk;
 use Tk::Text;
 use Tk::FileSelect;
+use Tk::DialogBox;
 use Pod::Text;
 use File::Basename;
 use Getopt::Long;
@@ -14,7 +15,7 @@ use Getopt::Long;
 # Global vars defined here
 
 my ($circle_frame, $delta, $fontlistbox, $fontentry, $scale, $text_frame);
-my ($fmw, $crw, $spw, $psfile, $viewer, $converter, $number_of_background);
+my ($fmw, $fgw, $fff, $crw, $caw, $spw, $psfile, $viewer, $converter, $number_of_background);
 my ($begin_from_bottom, $x_scale, $y_scale, $scale_x, $scale_y, $shift_x, $shift_y);
 my (@textfield_frame, @circle_text_frame, @textfield_entry, @entry, @text);
 my (@font, @fontsize, @begin_from_left, @eps_file_entry, @eps_filename, @eps_file_label);
@@ -25,6 +26,8 @@ my (@shift_eps_x, @shift_eps_y, @scale_eps_x, @scale_eps_y, @fontfamily, @fontna
 my (@color_rgb, @rgb, @ps_color, @tk_color, @circle_radius);
 
 my @configurationfile;
+my $fontpath;
+my %fonthash;
 my ($label_fn, $label_id);
 
 # Global parameters to be changed if needed
@@ -40,7 +43,6 @@ my @SaveVariables = qw(delta begin_from_bottom viewer converter
 		    circle_radius[0][2] circle_radius[0][3]
 		    circle_radius[1][0] circle_radius[1][1]
 		    circle_radius[1][2] circle_radius[1][3]
-		    shift_x shift_y scale_x scale_y
 		    x_scale y_scale
 		   );
 
@@ -74,11 +76,16 @@ my @LabelVariables = qw( text[0][0] text[0][1] text[0][2] text[0][3]
 		     color_rgb[1][6][0] color_rgb[1][6][1] color_rgb[1][6][2]
 		     color_rgb[1][7][0] color_rgb[1][7][1] color_rgb[1][7][2]
 		     color_rgb[1][9][0] color_rgb[1][9][1] color_rgb[1][9][2]
+		     shift_x shift_y scale_x scale_y starting_angle
 		     eps_file_label[0] eps_file_label[1]
 		     eps_filename[0] eps_filename[1]);
 
 my @FormVariables = qw(delta begin_from_bottom number_of_labels
-		    begin_from_left[0] begin_from_left[1]);
+		    circle_radius[0][0] circle_radius[0][1]
+		    circle_radius[0][2] circle_radius[0][3]
+		    circle_radius[1][0] circle_radius[1][1]
+		    circle_radius[1][2] circle_radius[1][3]
+            radius_inner begin_from_left[0] begin_from_left[1]);
 
 my $pi = 3.14159;
 my $home = $ENV{'HOME'};
@@ -86,11 +93,6 @@ my $home = $ENV{'HOME'};
 $configurationfile[0] = "/etc/cd-circleprint.conf";
 $configurationfile[1] = $home."/\.cd-circleprint.conf";
 
-defaults();
-default_attributes();
-default_text();
-default_colors();
-
 # read the command-line for any options
 
 my $params;
@@ -110,15 +112,47 @@ if ($params->{'version'}) {
 if ($params->{'conf_file'}) {
   $configurationfile[2] = $params->{'conf_file'};
 }
- 
-read_configuration();
 
-# Was a Label filename specified?
+# Build a list of available fonts and their Postscript names
 
-if (defined $ARGV[0]) {
-  $label_fn = $ARGV[0];
-  read_label();
+my $tmppath = qx'xset q | awk "/Font Path:/ { i=1 ; next } i==1 { print \$0 ; i=0 }"';
+while (defined ($tmppath)) {
+  ($fontpath, $tmppath) = split /,/, $tmppath, 2;
+  last if ($fontpath =~ /Type1/);
 }
+my $fontdir = $fontpath . "/fonts.dir";
+open (FONTD, "<$fontdir") ||
+    die "Can't Open font dir file: $!";
+my $strg = <FONTD>;
+$strg = <FONTD>; # skip the count record
+while (defined($strg)) {
+  my ($ffa, $ffb, $ffc, $ffd);
+  ($ffa, $ffb, $ffc) = split /-/, $strg;
+  $ffa =~ s/pfb/afm/;
+  my $fontfn = $fontpath . "/" . $ffa;
+  open (FONTA, "<$fontfn") ||
+      die "Can't Open font afm file: $!";
+  $strg = <FONTA>;
+  while (defined($strg)) {
+    chomp $strg;
+    ($ffb, $ffd) = split / /, $strg, 2;
+    last if ($ffb =~ /^FontName/);
+    $strg = <FONTA>;
+  }
+  close (FONTA);
+  $fonthash{$ffd} = $ffc;
+  $strg = <FONTD>;
+}
+close (FONTD);
+
+# set the system defaults
+
+defaults();
+default_attributes();
+default_text();
+default_colors();
+
+read_configuration();
 
 propogate_colors();
 
@@ -150,7 +184,7 @@ $menu_frame -> Menubutton(-text => "File",
 					  -command => \&open_label,
 					  -underline => 0],
 					 ['command' => "Save",
-					  -command => \&write_label,
+					  -command => \&save_label,
 					  -underline => 0],
 					 ['command' => "Save As",
 					  -command => \&save_as,
@@ -172,7 +206,7 @@ $menu_frame -> Menubutton(-text => "File",
 					  -underline => 0],
 					 "-",
 					 ['command' => "Exit",
-					  -command => sub{&clean_exit},
+					  -command => sub{&clean_exit;},
 					  -underline => 1]
 				       ]) -> pack(-side => 'left');
 
@@ -293,8 +327,6 @@ $textfield_frame[1] = $text_frame -> Frame(-label => "Top label",
 
 for ( my $j=0; $j < $number_of_labels; $j++) {
   for (my $i=0; $i < 4; $i++ ) {
-
-#           Entry Text
     $textfield_entry[$j][$i] = $textfield_frame[$j] -> Entry(-textvariable => \$text[$j][$i+4])
                                                     -> pack( -ipadx => 10, -ipady => 10 );
 
@@ -304,13 +336,8 @@ for ( my $j=0; $j < $number_of_labels; $j++) {
     $font[$j][$i] = $entry[$j][$i] -> fontCreate();
     $font[$j][$i+4] = $textfield_entry[$j][$i] -> fontCreate();
 
-    $which_field = $i;
-    $scale = $fontsize[$j][$i];
-    configure_fonts($j);
-    
-    $which_field = $i+4;
-    $scale = $fontsize[$j][$i+4];
-    configure_fonts($j);
+    configure_font($j, $i);
+    configure_font($j, $i+4);
   }
 }
 
@@ -324,6 +351,14 @@ $textfield_frame[0] -> Button(-text => "clear",
 $textfield_frame[1] -> Button(-text => "clear",
 				-command => sub{clear_textfields (1)} ) -> pack();
 
+# Was a Label filename specified?
+
+if (defined $ARGV[0]) {
+  $label_fn = $ARGV[0];
+  read_label();
+  reconfigure_fonts();
+}
+
 update_display();
 
 ############################
@@ -339,27 +374,28 @@ sub clean_exit {
   if ($pidsav != 0 ) {
     `kill -USR1 $pidsav`;
   }
-  exit 0;
+
+  exit;
 
 }
 #######################
 # Very incomplete :-(
 #
-sub create_canvas {
+#sub create_canvas {
 # First we need a new Window to show the objects
 
- my $canvas;
+# my $canvas;
 
- my $cmw = MainWindow -> new;
+# my $cmw = $mw->Toplevel();
 
-  $cmw -> title("Move oobjects of the labels");
-  $canvas = $cmw -> Scrolled('Canvas')->pack(  );
+#  $cmw -> title("Move oobjects of the labels");
+#  $canvas = $cmw -> Scrolled('Canvas')->pack(  );
 
 # The inner circle
-my $inner_crcle = $canvas->createOval(10,10,100,100, -extent => 360);
+#my $inner_crcle = $canvas->createOval(10,10,100,100, -extent => 360);
 
 
-}
+#}
 
 #######################
 sub add_font_from_entry {
@@ -368,6 +404,17 @@ sub add_font_from_entry {
   push(@fontnames,$fontentry -> get());
 }
 
+#######################
+sub reconfigure_fonts {
+
+  for ( my $j=0; $j < $number_of_labels; $j++) {
+    for (my $i=0; $i < 4; $i++ ) {
+      configure_font($j, $i);
+      configure_font($j, $i+4);
+    }
+  }
+}
+
 #######################
 sub open_label {
 
@@ -379,10 +426,10 @@ sub open_label {
                              -filter => '[^.]*', -acceptlabel => 'Ok');
   $olfile = $OLref -> Show;
 
-  if (defined $olfile) {
+  if ((defined $olfile) && ($olfile ne "")) {
     $label_fn = $olfile;
     read_label();
-    propogate_colors();
+    reconfigure_fonts();
     update_display();
   }
 }
@@ -424,27 +471,43 @@ sub clear_textfields {
   }
 }
 #######################
-sub configure_fonts {
-  # It is not possible to use the size from the perl-tk fonts, because perl-tk
-  # uses the nearest matching font on the system but we want exact PostScript fonts
+sub configure_font {
 
+  my ($ffamily, $fweight, $fslant);
   my $j = shift;
+  my $i = shift;
 
-  $fontsize[$j][$which_field] =   $scale ;
+  $ffamily = $fonthash{$fontfamily[$j][$i]};
+  $fweight = "normal";
+  $fslant = "roman";
 
-  if ($which_field < 4) {
-    $entry[$j][$which_field] -> fontConfigure($font[$j][$which_field],
-					       -size => $scale,
-					       -family => $fontfamily[$j][$which_field]);
+  if (($fontfamily[$j][$i] =~ /bold/i) ||
+      ($fontfamily[$j][$i] =~ /dark/i)) {
+    $fweight = "bold";
+  }
 
-    $entry[$j][$which_field] -> configure(-font => $font[$j][$which_field]);
+  if (($fontfamily[$j][$i] =~ /italic/i) ||
+      ($fontfamily[$j][$i] =~ /oblique/i)) {
+    $fslant = "italic";
+  }
+
+  if ($i < 4) {
+    $entry[$j][$i] -> fontConfigure($font[$j][$i],
+					       -size => $fontsize[$j][$i],
+                           -weight => $fweight,
+                           -slant => $fslant,
+					       -family => $ffamily);
+
+    $entry[$j][$i] -> configure(-font => $font[$j][$i]);
   }
   else {
-    $textfield_entry[$j][$which_field-4] -> fontConfigure($font[$j][$which_field],
-					       -size => $scale,
-					       -family => $fontfamily[$j][$which_field]);
+    $textfield_entry[$j][$i-4] -> fontConfigure($font[$j][$i],
+					       -size => $fontsize[$j][$i],
+                           -weight => $fweight,
+                           -slant => $fslant,
+					       -family => $ffamily);
 
-    $textfield_entry[$j][$which_field-4] -> configure(-font => $font[$j][$which_field]);
+    $textfield_entry[$j][$i-4] -> configure(-font => $font[$j][$i]);
   }
 }
 
@@ -598,15 +661,16 @@ sub check_viewer {
 sub default_attributes {
   # fontsizes
   @fontsize = ( [qw (32 18 18 32 16 16 16 16) ], [qw (32 18 18 32 16 16 16 16)]);
-  $scale = $fontsize[0][0]; # initial value
 
-# This should be replaced by a routine which looks for all available fonts
-# Any volunteers?
-  @fontnames = qw(AvantGard AvantGard-BookOblique AvantGard-Demi AvantGard-DemiOblique Helvetica Helvetica-Oblique Helvetica-Bold Helvetica-BoldOblique Bookman-Demi Courier Palatino-Roman Times-Roman NewCenturySchlbk-Italic NewCenturySchlbk-BoldItalic NewCenturySchlbk-Italic NewCenturySchlbk-Roman);
+  my $ctr = 0;
+  foreach my $ffc (sort keys %fonthash) {
+    $fontnames[$ctr] = $ffc;
+    $ctr++;
+  }
 
   for (my $j=0; $j < $number_of_labels; $j++ ) {
     for (my $i=0; $i < 8; $i++ ) {
-      $fontfamily[$j][$i] = "Times-Roman" ;
+      $fontfamily[$j][$i] = $fontnames[0];
     }
   }
 }
@@ -659,7 +723,7 @@ sub modify_color {
 
 # make the color selector window follow the chosen color
 
-  $fmw -> configure(-background => $tk_color[$label_id][$which_field]);
+  $fff -> configure(-background => $tk_color[$label_id][$which_field]);
   
   update_display();
 }
@@ -698,7 +762,7 @@ sub set_selectors {
 
 #   display the right entry in the fontlistbox
     foreach (0 .. $#fontnames ) {
-      if ( $fontnames[$_] =~ /$fontfamily[$label_id][$which_field]/) {
+      if ( $fontnames[$_] eq $fontfamily[$label_id][$which_field]) {
 	$fontlistbox -> selectionClear(0, "end");
 	$fontlistbox -> selectionSet($_);
 	$fontlistbox -> see($_);
@@ -711,7 +775,7 @@ sub set_selectors {
   $rgb[2] = $color_rgb[$label_id][$which_field][2];
 
 # and the colour selector background
-  $fmw -> configure(-background => $tk_color[$label_id][$which_field]);
+  $fff -> configure(-background => $tk_color[$label_id][$which_field]);
 }
 
 ########################
@@ -736,9 +800,8 @@ sub help_window {
   my $tmp_file_name = "/tmp/cd-circle_print.man.1";
   $parser->parse_from_file ($0, $tmp_file_name);
 
-  my $help_window = MainWindow -> new;
-  my $text = $help_window -> Scrolled("Text", -scrollbars => 'oe') -> pack;
-#  $help_window -> reconfigure(-font => "r16");
+  my $help_dialog = $mw->DialogBox(-title => 'Help', -buttons => ['OK'], -default_button => 'OK');
+  my $text = $help_dialog -> Scrolled("Text", -scrollbars => 'oe') -> pack;
 
   open (TMP_FILE,"<$tmp_file_name") or (return());
   while ( <TMP_FILE>) {
@@ -747,13 +810,18 @@ sub help_window {
   close (TMP_FILE);
 # We should delete the tmp-file
 
-  $help_window -> Button(-text => "close", 
-			 -command => sub {$help_window -> destroy() if Tk::Exists($help_window)}) -> pack;
+# fix needed to get around locking waitVariable() call, which is never unblocked when the dialog is destroyed.
+  $help_dialog->OnDestroy(sub { $help_dialog->{'selected_button'} = $help_dialog->{'selected_button'} });
+
+  $help_dialog->Show();
 }
+
 ########################
 sub about_window {
-  my $about_window = MainWindow -> new;
-  my $text = $about_window -> Text(-font => "r12") -> pack;
+
+  my $about_box = $mw->DialogBox(-title => 'About', -buttons => ['OK'], -default_button => 'OK');
+
+  my $text = $about_box -> Text() -> pack;
 
   $text -> insert('end', "cd-circleprint version: $VERSION\n\n");
   $text -> insert('end', "a simple perl script by Colin Tuckley <colintu\@users.sourceforge.net>\n");
@@ -767,8 +835,10 @@ sub about_window {
   $text -> tagConfigure('red', -foreground => "#ff0000");
   $text -> insert('end',"May the source be with you", 'red');
 
-  $about_window -> Button(-text => "close",
-			 -command => sub {$about_window -> destroy() if Tk::Exists($about_window)}) -> pack;
+# fix needed to get around locking waitVariable() call, which is never unblocked when the dialog is destroyed.
+  $about_box->OnDestroy(sub { $about_box->{'selected_button'} = $about_box->{'selected_button'} });
+
+  $about_box->Show();
 }
 
 ########################
@@ -786,6 +856,7 @@ sub read_label {
     }
   }
   set_graphics_attributes();
+  propogate_colors();
 }
 
 ########################
@@ -838,7 +909,7 @@ $FSref = $mw -> FileSelect(-directory => $home, -title => 'Read Form Data',
                            -filter => '[^.]*', -acceptlabel => 'Ok');
 my $fdfile = $FSref -> Show;
 
-  if (defined $fdfile) {
+  if ((defined $fdfile) && ($fdfile ne "")) {
     open (FD_FILE, "<$fdfile") || die ("Could not open $fdfile\n");
     while (<FD_FILE>) {
       next unless /(^\w+.*)\s+(.*$)/;
@@ -916,19 +987,19 @@ sub print_circles {
     print PS_FILE "newpath\n";
 
     # Then we print the graphics
-    if ($eps_file_label[$label_no]) {
-	# check wether it is an eps-file
-	$epsname =`file $eps_filename[$label_no]`;
-	if ($epsname =~ /EPS/) {
+    if (($eps_file_label[$label_no]) && ($eps_filename[$label_no] ne "")) {
+	  # check wether it is an eps-file
+	  $epsname =`file $eps_filename[$label_no]`;
+	  if ($epsname =~ /EPS/) {
 	    print_graphics($eps_filename[$label_no],$label_no);
-	}
-	else {
+	  }
+	  else {
 	    $epsname = $eps_filename[$label_no];
 	    ($name,$dir,$ext) = fileparse($epsname);
 	    $epsname = "/tmp/".$name.".eps";
 	    system "$converter $eps_filename[$label_no] $epsname";
 	    print_graphics($epsname, $label_no);
-	}
+	  }
     }
 
     # After that we need the small inner circle in white
@@ -1042,6 +1113,7 @@ sub print_label{
   print PS_FILE "showpage\n";
   print PS_FILE "%%Trailer\n";
   print PS_FILE "%%Pages: 1\n";
+  print PS_FILE "%%EOF\n";
 
   close (PS_FILE);
 }
@@ -1478,6 +1550,7 @@ sub get_bounding_box() {
 ########################
 sub get_eps_file {
   my $FSref;
+  my $fn;
   my $epsfile;
   my $i;
 
@@ -1488,53 +1561,55 @@ sub get_eps_file {
 # set the filename if defined as default !!!!!!!!!!!
   $FSref = $mw -> FileSelect(-directory => $graphics_directory, -title => 'EPS File',
 #			     -defaultextension => "eps",
- 			     -acceptlabel => 'Ok'
+ 			     -acceptlabel => 'Ok',
 			     -filter => '[^.]*', -initialfile => $epsfile);
 
-  $graphics_file = $FSref -> Show;
+  $fn = $FSref -> Show;
 
-  if ( defined $graphics_file) {
-# Remember the path 
-# First get it from the string
-  $graphics_file =~ /(.*\/)(.*)/;
-# and store it  
-  $graphics_directory = $1;
+  if ((defined $fn) && ($fn ne "")) {
+    $graphics_file = $fn;
+#   Remember the path 
+#   First get it from the string
+    $graphics_file =~ /(.*\/)(.*)/;
+#   and store it  
+    $graphics_directory = $1;
     for ( $i=0; $i < $number_of_labels; $i++) {
       if ($eps_file_label[$i]) {
-	$eps_filename[$i] = $graphics_file;
-	$eps_file_entry[$i] -> configure(-textvariable => \$graphics_file);
+	    $eps_filename[$i] = $graphics_file;
+	    $eps_file_entry[$i] -> configure(-textvariable => \$graphics_file);
       }
     }
   }
-  return $graphics_file;
 }
 
 ########################
 sub get_fontattributes {
 
-  my $fwb; #Frame for buttons
   my $flb;
   my $fw; #Frame
   my ($flco, $i);
 
-  $fmw = MainWindow -> new;
+  if (! Exists($fmw)) {
+  $fmw = $mw->Toplevel();
 
   $fmw -> title("FONTS, size and colours");
 
-# One frame for Ok and Close-button
+  $fmw->Button(-text => "OK", -command => sub{$fmw->withdraw})->pack(-side => 'bottom');
 
-  $fwb = $fmw -> Frame(-relief => 'raised', -bd => 2) -> pack(-side => 'bottom',
-						      -fill => 'both',
-						      -anchor => 's',
-						      -expand => '1');
+# A frame for everything except the ok button
 
-# Now we want a frame for the size and the radiobuttons
-  $fw = $fmw -> Frame(-relief => 'raised', -bd => 2) -> pack(-side => 'left',
+  $fff = $fmw -> Frame() -> pack(-side => 'top');
+
+# Now we want a frame for the radiobuttons
+  $fw = $fff -> Frame(-relief => 'raised', -bd => 2) -> pack(-side => 'left',
 						      -fill => 'both',
 						      -anchor => 'e',
 						      -expand => '1');
 
-# This frame is divided into two frames
+# This frame is divided into three frames
+
+  my $fwb = $fw -> Frame() -> pack(-side => 'bottom',
+				-fill => 'both');
 
   my $fwc = $fw -> Frame() -> pack(-side => 'left',
 				-fill => 'both');
@@ -1614,68 +1689,46 @@ sub get_fontattributes {
                               -anchor => 'w',
 			    -side => 'top');
 
-  $fw -> Radiobutton(-text => "Background",
+  $fwb -> Radiobutton(-text => "Background",
 		     -value => $number_of_background,
 		     -variable => \$which_field,
 		     -command => sub{set_selectors()}
-		     ) ->pack(-expand => 'x',
-			       -anchor => 'w');
-
-  $fw -> Scale(-from => 8,
-	       -to => 44,
-	       -orient => 'horizontal',
-	       -variable => \$scale,
-	       -command => sub{modify_fonts() } ) -> pack(-side => 'right',
-							-expand => 'x') ;
+		     ) ->pack(-expand => 'x', -padx => 100,
+			   -pady => 10, -anchor => 'w');
 
 ############################################################
-# Now stop the frame and put the close-button in the window
+# Another frame for the fontlist and size
 ############################################################
 
-  $fwb -> Button(-text => "close",
-		 -command => sub{$fmw -> destroy() if Tk::Exists($fmw)}) -> pack;
-
-############################################################
-# Another frame for the fontlist
-############################################################
-
-  $flb = $fmw -> Frame(-relief => 'raised', -bd => 2) -> pack(-side => 'left',
+  $flb = $fff -> Frame(-relief => 'raised', -bd => 2) -> pack(-side => 'left',
 							      -fill => 'x',
 							      -anchor => 'e',
 							      -expand => '1');
 
-  $fontlistbox = $flb -> Scrolled("Listbox", -scrollbars => "se", -selectmode => "single") -> pack();
-
-############################################################
-# we should have these fonts with ghostscript
-# feel free to write a subroutine to fill the listbox with fonts that really exist on the local machine
-############################################################
+  $fontlistbox = $flb -> Scrolled("Listbox", -scrollbars => "e", -selectmode => "single", -width => 0) -> pack();
 
   foreach $i  ( @fontnames) {
     $fontlistbox -> insert('end', $i);
   }
 
   $fontlistbox -> bind('<Button-1>',
-		      sub{$fontfamily[$label_id][$which_field] =  $fontnames[$fontlistbox -> curselection];
+		      sub{$fontfamily[$label_id][$which_field] =  $fontnames[$fontlistbox->curselection->[0]];
 			   modify_fonts() }
 		      );
 
-  $fontlistbox -> bind('<Button-3>',
-		       sub{$fontlistbox -> delete($fontlistbox -> curselection);});
-
-  $flb -> Button(-text => "Delete selected",
-		-command => sub{$fontlistbox -> delete($fontlistbox -> curselection)}) -> pack(-side => 'top');
-
-  $flb -> Label(-text => "Additional fonts:") -> pack();
-  $fontentry = $flb -> Entry(-background => "#ffffff") -> pack();
-  $flb -> Button(-text => "Add",
-		 -command => \&add_font_from_entry) -> pack(-side => 'left');
+  $flb -> Scale(-from => 8,
+	       -to => 44,
+	       -orient => 'horizontal',
+	       -variable => \$scale,
+	       -command => sub{$fontsize[$label_id][$which_field] = $scale;
+                           modify_fonts() } ) -> pack(-side => 'right',
+							-expand => 'x') ;
 
 #####################################
 # Next we want a frame for colors
 #####################################
 
-my $fco = $fmw -> Frame(-relief => 'raised', -bd => 2) -> pack(-side => 'left',
+my $fco = $fff -> Frame(-relief => 'raised', -bd => 2) -> pack(-side => 'left',
 							    -fill => 'x',
 							    -anchor => 'e',
 							    -expand => '1');
@@ -1683,109 +1736,120 @@ my $fco = $fmw -> Frame(-relief => 'raised', -bd => 2) -> pack(-side => 'left',
 $fco -> bind("<ButtonRelease>",\&modify_color());
 
 $fco ->  Scale(-label => "red",
+           -length => 140,
 	       -variable => \$rgb[0],
 	       -resolution => 1,
 	       -command => sub{modify_color()},
 	       -from => 255, -to => 0) -> pack(-side => 'left');
 
 $fco ->  Scale(-label => "green",
+           -length => 140,
 	       -variable => \$rgb[1],
 	       -resolution => 1,
 	       -command => sub{modify_color()},
 	       -from => 255, -to => 0) -> pack(-side => 'left');
 
 $fco ->  Scale(-label => "blue",
+           -length => 140,
 	       -variable => \$rgb[2],
 	       -resolution => 1,
 	       -command => sub{modify_color()},
 	       -from => 255, -to => 0) -> pack(-side => 'left');
+} else {
+  $fmw->deiconify();
+  $fmw->raise;
 }
+
+set_selectors();
+
+}
+
 ########################
 sub get_graphics {
   my $fwb; #Frame for labelbuttons
-  my $fwcb; #Frame for close button
   my $fwf; #Frame for the files
   my $fwg; #Frame for graphics-place buttons
   my $fw; #Frame for the radiobuttons
   my ($fwscale, $fwshift);
+  my ($ox, $oy);
   my $i;
 
-  $fmw = MainWindow -> new;
+  if (! Exists($fgw)) {
+    $fgw = $mw->Toplevel();
 
-  $fmw -> title("Graphics files");
+    $fgw -> title("Graphics files");
+
+    $fgw->Button(-text => "OK", -command => sub{$fgw->withdraw})->pack(-side => 'bottom');
 
 # One frame for the labels
-  $fwb = $fmw -> Frame(-relief => 'raised', -bd => 2) -> pack(-side => 'top',
+    $fwb = $fgw -> Frame(-relief => 'raised', -bd => 2) -> pack(-side => 'top',
 							      -fill => 'both',
 							      -anchor => 'n',
 							      -expand => '1');
 
 # One more frame for the Files
-  $fwf = $fmw -> Frame(-relief => 'raised', -bd => 2) -> pack(-side => 'top',
+    $fwf = $fgw -> Frame(-relief => 'raised', -bd => 2) -> pack(-side => 'top',
 							       -fill => 'both');
 
 # One more for shifting
-  $fwshift = $fmw -> Frame(-relief => 'raised', -bd => 2) -> pack(-side => 'top',
+    $fwshift = $fgw -> Frame(-relief => 'raised', -bd => 2) -> pack(-side => 'top',
 							       -fill => 'both');
 # One more for scaling
-  $fwscale = $fmw -> Frame(-relief => 'raised', -bd => 2) -> pack(-side => 'top',
+    $fwscale = $fgw -> Frame(-relief => 'raised', -bd => 2) -> pack(-side => 'top',
 							       -fill => 'both');
 
-# One frame for Ok and Close-button
-  $fwcb = $fmw -> Frame(-relief => 'raised', -bd => 2) -> pack(-side => 'top',
-							       -fill => 'both',
-							       -expand => '1');
-
-  $fwb -> Checkbutton(-text => "Upper label",
+    $fwb -> Checkbutton(-text => "Upper label",
 		     -variable => \$eps_file_label[1],
 		     -command => sub{set_graphics_attributes()}) -> pack(-side => 'right',
 						       -expand => 'x',
 						       -anchor => 'w');
-  $fwb -> Checkbutton(-text => "Lower label",
+    $fwb -> Checkbutton(-text => "Lower label",
 		     -variable => \$eps_file_label[0],
 		     -command => sub{set_graphics_attributes()}) -> pack(-side => 'right',
 						       -expand => 'x',
 						       -anchor => 'w',);
 
-  for ($i=0; $i < $number_of_labels; $i++) {
-    $eps_file_entry[$i] = $fwf -> Entry(-text => "EPS-File $i",
-					-textvariable => \$eps_filename[$i],
+    for ($i=0; $i < $number_of_labels; $i++) {
+      $eps_file_entry[$i] = $fwf -> Entry( -textvariable => \$eps_filename[$i],
 					-background => "#ffffff") -> pack(-side => 'bottom');
-  }
+    }
 
-  $fwf -> Button(-text => "browse",
+    $fwf -> Button(-text => "browse",
 		 -command => sub{get_eps_file()}) -> pack();
 
-  $fwshift -> Scale(-label => "Offset X",
-		    -from => -$radius_outer, -to => $radius_outer,
+    $ox = $fwshift -> Scale(-label => "Offset X",
+		    -from => -$radius_outer/$scale_x, -to => $radius_outer/$scale_x,
 		    -resolution => "0.1",
 		    -orient => "horizontal",
 		    -command => sub{set_graphics_attributes()},
 		    -variable => \$shift_x) -> pack(-side => "left");
 
-  $fwshift -> Scale(-label => "Offset Y",
-		    -from => $radius_outer, -to => -$radius_outer,
+    $oy = $fwshift -> Scale(-label => "Offset Y",
+		    -from => $radius_outer/$scale_y, -to => -$radius_outer/$scale_y,
 		    -resolution => "0.1",
 		    -orient => "vertical",
 		    -command => sub{set_graphics_attributes()},
 		    -variable => \$shift_y) -> pack(-side => "right");
 
-  $fwscale -> Scale(-label => "Scale X",
-		    -from => 0, -to => 1,
+    $fwscale -> Scale(-label => "Scale X",
+		    -from => 0.05, -to => 1,
 		    -resolution => "0.01",
 		    -orient => "horizontal",
-		    -command => sub{set_graphics_attributes()},
+		    -command => sub{$ox->configure(-from => -$radius_outer/$scale_x,
+                                           -to => $radius_outer/$scale_x); set_graphics_attributes()},
 		    -variable => \$scale_x) -> pack(-side => "left");
 
-  $fwscale -> Scale(-label => "Scale Y",
-		    -from => 1, -to => 0,
+    $fwscale -> Scale(-label => "Scale Y",
+		    -from => 1, -to => 0.05,
 		    -resolution => "0.01",
 		    -orient => "vertical",
-		    -command => sub{set_graphics_attributes()},
+		    -command => sub{$oy->configure(-from => $radius_outer/$scale_y,
+                                           -to => -$radius_outer/$scale_y); set_graphics_attributes()},
 		    -variable => \$scale_y) -> pack(-side => "right");
-
-  $fwcb -> Button(-text => "close",
-		  -command => sub{$fmw -> destroy() if Tk::Exists($fmw)}) -> pack(-side => 'bottom');
+  } else {
+    $fgw->deiconify();
+    $fgw->raise;
+  }
 }
 
 ########################
@@ -1797,7 +1861,7 @@ sub get_converter {
                                -acceptlabel => 'Ok');
     $fc = $FSref -> Show;
 
-    $converter = $fc if defined $fc;
+    $converter = $fc if ((defined $fc) && ($fc ne ""));
 }
 
 ########################
@@ -1809,27 +1873,40 @@ sub get_previewer {
                              -acceptlabel => 'Ok');
   $pv = $FSref -> Show;
 
-  if (defined $pv) {
+  if ((defined $pv) && ($pv ne "")) {
     check_viewer($pv);
   }
 }
 ########################
 sub get_starting_angle {
 
-  my $starting_angle_window;
+if (! Exists($caw)) {
+    $caw = $mw -> Toplevel();
+    $caw -> title("Angle to middle of text");
+    $caw -> Button(-text => "OK",
+		   -command => sub {$caw -> withdraw}) -> pack(-side => 'bottom');
 
-  $starting_angle_window = MainWindow -> new;
-  $starting_angle_window -> geometry("+100+100");
-
-  $starting_angle_window -> Scale(-label => "Angle to middle of text",
+  $caw -> Scale(-label => "Angle to middle of text",
 				  -from => -90, -to => 90,
 				  -resolution => "0.5",
 				  -orient => "horizontal", -length => 250,
 				  -variable => \$starting_angle) -> pack(-side => "top");
+  }
+  else {
+    $caw -> deiconify();
+    $caw -> raise();
+  }
+}
 
-  $starting_angle_window -> Button(-text => "Ok",
-				   -command => sub {$starting_angle_window -> destroy()
-						      if Tk::Exists($starting_angle_window)}) -> pack(-side => "bottom");
+########################
+sub save_label {
+
+  if (defined $label_fn) {
+    write_label();
+  }
+  else {
+    save_as();
+  }
 }
 
 ########################
@@ -1843,7 +1920,7 @@ sub save_as {
                              -filter => '[^.]*', -acceptlabel => 'Ok');
   $safile = $SAref -> Show;
 
-  if (defined $safile) {
+  if ((defined $safile) && ($safile ne "")) {
     $label_fn = $safile;
     write_label();
   }
@@ -1853,13 +1930,15 @@ sub save_as {
 sub save_ps_file {
 
   my $FSref;
+  my $filen;
   my $home = $ENV{'HOME'};
 
   $FSref = $mw -> FileSelect(-directory => $home, -title => 'Save PS File',
                              -filter => '[^.]*', -acceptlabel => 'Ok');
-  $psfile = $FSref -> Show;
+  $filen = $FSref -> Show;
 
-  if (defined $psfile) {
+  if ((defined $filen) && ($filen ne "")) {
+    $psfile = $filen;
     print_label;
   }
 }
@@ -1867,10 +1946,10 @@ sub save_ps_file {
 ########################
 sub set_circle_radius {
 
-my (@lf, $tf, $bf);
+my (@lf, @lsc, $tf, $bf, $i, $j);
 
 if (! Exists($crw)) {
-    my $crw = $mw -> Toplevel();
+    $crw = $mw -> Toplevel();
     $crw -> title("Radii");
     $crw -> Button(-text => "OK",
 		   -command => sub {$crw -> withdraw}) -> pack(-side => 'bottom');
@@ -1880,17 +1959,19 @@ if (! Exists($crw)) {
     $tf -> Scale(-orient => 'horizontal',
 		  -variable => \$radius_inner,
 		  -from => 0, -to => 50,
-		  -resolution => "0.5",) -> pack(-side => 'left');
+		  -resolution => "0.5",
+          -command => sub { for ($j=0; $j<2 ; $j++) { for ($i=0; $i<4 ; $i++) {
+                      $lsc[$j][$i]->configure(-from => $radius_inner)}}}) -> pack(-side => 'left');
 
     $bf = $crw -> Frame(-label => "Radius of the text-circles [mm]") -> pack(-side => 'bottom');
     # Now the radii for the text circles
 
     my @side = qw (bottom right);
 
-    for (my $j=0; $j<2 ; $j++) {
-      $lf[$j] = $bf -> Frame(-label => "Circle $j", -borderwidth => "5", -relief => 'groove') -> pack(-side => 'left');
-      for ( my $i=0; $i<4 ; $i++) {
-	$lf[$j] -> Scale(-orient => 'horizontal',
+    for ($j=0; $j<2 ; $j++) {
+      $lf[$j] = $bf -> Frame(-label => "Label $j", -borderwidth => "5", -relief => 'groove') -> pack(-side => 'left');
+      for ($i=0; $i<4 ; $i++) {
+        $lsc[$j][$i] = $lf[$j] -> Scale(-orient => 'horizontal',
 		     -variable => \$circle_radius[$j][$i],
 		     -from => $radius_inner, -to => $radius_outer,
 		     -resolution => "0.5",) -> pack(-side => 'top');
@@ -1902,13 +1983,14 @@ if (! Exists($crw)) {
     $crw -> raise();
   }
 }
+
 ########################
 sub modify_fonts {
 
-  $fontsize[$label_id][$which_field] = $scale;
-  if ( $which_field < $number_of_background) {  configure_fonts($label_id);  }
+  if ( $which_field < $number_of_background) {  configure_font($label_id, $which_field);  }
   update_display();
 }
+
 ########################A
 sub set_graphics_attributes {
 
@@ -1959,7 +2041,7 @@ sub set_positions {
 
   if (! Exists($spw)) {
     my $spw = $mw -> Toplevel();
-    $spw -> title("Change circle positions to fit the sheet");
+    $spw -> title("Change label positions to fit the sheet");
     $spw -> Button(-text => "OK",
 		   -command => sub {$spw -> withdraw}) -> pack(-side => 'bottom');
     $spw -> Label(-text => "Use the tickmark scales to determine the positions",
@@ -1973,8 +2055,8 @@ sub set_positions {
     for (my $label_no = 0; $label_no < $number_of_labels; $label_no++ ) {
       $fwt -> Entry(-textvariable => \$begin_from_left[$label_no],
 		    -text => "Labeltext",
-		   -width => '5') -> pack(-side => 'bottom');
-      $fwt -> Scale(-label => "move circle $label_no horizontaly",
+		    -width => '5') -> pack(-side => 'bottom');
+      $fwt -> Scale(-label => "move label $label_no horizontaly",
 		    -orient => 'horizontal',
 		    -length => 250,
 		    -resolution => 0.1,
@@ -1999,7 +2081,7 @@ sub set_positions {
 		 -from => 200, -to => -50 ) -> pack(-side => 'left');
 
 
-    $fw -> Scale(-label => "distance between the circles",
+    $fw -> Scale(-label => "distance between the labels",
 		 -variable => \$delta,
 		 -resolution => 0.1,
 		 -from => 100, -to => 0 ) -> pack(-side => 'right');
@@ -2019,14 +2101,15 @@ sub set_positions {
 sub warning_window {
 
   my $message = shift;
-  my $warning_main_window = MainWindow -> new;
+  my $warning_box = $mw->DialogBox(-title => 'Warning', -buttons => ['OK'], -default_button => 'OK');
 
-  my $text = $warning_main_window -> Label(-text => $message,
+  my $text = $warning_box -> Label(-text => $message,
 					  -background => '#ff0000') -> pack;
 
-  $warning_main_window -> Button(-text => "Ok",
-				 -command => sub {$warning_main_window -> destroy() 
-						    if Tk::Exists($warning_main_window)}) -> pack;
+# fix needed to get around locking waitVariable() call, which is never unblocked when the dialog is destroyed.
+  $warning_box->OnDestroy(sub { $warning_box->{'selected_button'} = $warning_box->{'selected_button'} });
+
+  $warning_box->Show();
 
 }
 
@@ -2087,8 +2170,8 @@ $FSref = $mw -> FileSelect(-directory => $home, -title => 'Form Data',
                            -filter => '[^.]*', -acceptlabel => 'Ok');
 my $fdfile = $FSref -> Show;
 
-  if (defined $fdfile) {
-	  
+  if ((defined $fdfile) && ($fdfile ne "")) {
+
     open (FD_FILE, ">$fdfile") || die ("Could not open $fdfile\n");
 
     my $date = localtime;
@@ -2122,8 +2205,6 @@ sub dummysub {
 ########################################
 #
 # Documentation
-# Needs much more
-# Any english speaking volunteers?
 #
 #######################################
 
@@ -2152,7 +2233,7 @@ You can select the font, size, and colour of the text for each field separately,
 can choose the background colour for each label. An image file in EPS format may also be used as a
 background.  If you wish more complicated backgrounds
 or colour effects you can save a label and modify it in any graphic program which can read
-PostScript. (I suggest "The Gimp")
+PostScript.
 
 =back
 
@@ -2195,7 +2276,7 @@ You can also squeeze or stretch the whole thing to correct small errors of your
 =head1 USAGE
 
 After starting the program you see sixteen text-entry fields divided into two blocks. 
-The upper block is for the straight text fields, the lower block is for the arcs.
+The upper block is for the arcs, the lower block is for the straight text fields.
 
 Each label has four text fields and four arcs.
 
@@ -2203,8 +2284,8 @@ The first two lines are for the upper arcs on the label and the last two for the
 
 Write your text in the fields (support for cddb is on the TODO list)
 
-The Save Image menu item creates a file
-/tmp/cd-circleprint.ps which can be previewed with a PostScript viewer.
+The Save Image menu item creates a postscript file of the label.
+A file selection dialogue is displayed so the user can specify the path and filename.
 
 Font attributes and colours can be changed interactively using a dialog available from the menu.
 
@@ -2219,7 +2300,7 @@ Font attributes and colours can be changed interactively using a dialog availabl
 
 =over
 
-=item I<S<New >>
+=item I<New>
 
 Start a new label, all label fields are cleared and parameters are reset to default values.
 
@@ -2268,7 +2349,7 @@ Exit the program, all unsaved data will be lost.
 Change font family, font size, font-colour and the background colour interactively, all
 changes should take effect immediately in the text entry fields.
 
-Be carefull, perl-tk uses the nearest match for the font. The labels may look different on your printer.
+Note, perl-tk uses the nearest match for the font. The labels may look different on your printer.
 
 =item I<Angle>
 
@@ -2367,8 +2448,7 @@ or selected from a file dialog.
 
 =head1 BUGS
 
-There are probably many bugs and much ugly code but it works.
-Please fix them and send me a patch.
+There are probably many bugs, please fix them and send me a patch.
 Thanks
 
 =head1 AUTHORS
